#define BOOST_TEST_MODULE testcase for fileio
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <fstream>
#include <boost/shared_ptr.hpp>
#include <harkio2/harkconfig/HarkConfigFileTextReader.hpp>
#include <harkio2/harkconfig/HarkConfigFileTextHelper.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFileTextHelper.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/Version.hpp>
#include <harkio2/AnalyzeError.hpp>
#include "ImpulseResponseListFileData.hpp"

using namespace std;
using namespace boost;
using namespace harkio2;
using namespace harkio2::harkconfig;

BOOST_AUTO_TEST_CASE(shrink_test)
{
    BOOST_CHECK("hoge" != string(" \thoge"));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink("hoge"));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink(" \thoge"));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink("hoge  \t"));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink(" \t hoge \t "));
    BOOST_CHECK_EQUAL("", HarkConfigFileTextReader::Shrink(" \t \t "));
}

BOOST_AUTO_TEST_CASE(comment_test)
{
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::EliminateComment("hoge"));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::EliminateComment("hoge#comment"));
    BOOST_CHECK_EQUAL("", HarkConfigFileTextReader::EliminateComment("#comment"));
}

BOOST_AUTO_TEST_CASE(comment_shrink_test)
{
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink(HarkConfigFileTextReader::EliminateComment("hoge")));
    BOOST_CHECK_EQUAL("hoge", HarkConfigFileTextReader::Shrink(HarkConfigFileTextReader::EliminateComment(" hoge # comment ")));
    BOOST_CHECK_EQUAL("", HarkConfigFileTextReader::Shrink(HarkConfigFileTextReader::EliminateComment(" \t # comment ")));
}

BOOST_AUTO_TEST_CASE(GetValidLine_test)
{
    stringstream ss;
    ss <<
        "# ImpulseResponse-ListFile \n"
        "# DateTime; 2010/12/14 12:14:03\n"
        " ImpulseResponse-ListFile \n"
        "\n"
        "\tVersion = 1.0\n"
        ;

    string line;
    HarkConfigFileTextReader::GetValidLine(ss, line);
    BOOST_CHECK_EQUAL("ImpulseResponse-ListFile", line);
    HarkConfigFileTextReader::GetValidLine(ss, line);
    BOOST_CHECK_EQUAL("Version = 1.0", line);
}

BOOST_AUTO_TEST_CASE(ImpulseResponse1_0Helper_test)
{
    ImpulseResponseListFile1_0TextHelper helper;

    boost::shared_ptr<HarkConfigFile> conf = helper.GetData();

    BOOST_CHECK(conf);
    if (conf)
    {
        BOOST_CHECK_EQUAL(conf->GetId(), "ImpulseResponseListFile");
        BOOST_CHECK(conf->GetVersion() == Version(1, 0));
    }

    boost::shared_ptr<ImpulseResponseListFile1_0> obj = static_pointer_cast<ImpulseResponseListFile1_0>(conf);

    BOOST_CHECK(obj);
    {
        BOOST_CHECK_EQUAL(obj->GetSources().size(), 0);
    }

    helper.AnalyzeLine("1, 2, 3 %%hoge, 4");
    conf = helper.GetData();
    obj = static_pointer_cast<ImpulseResponseListFile1_0>(conf);

    BOOST_CHECK(obj);
    if (obj)
    {
        BOOST_CHECK_EQUAL(obj->GetSources().size(), 1);
        BOOST_CHECK(obj->GetSources()[0].GetPos() == Vector3<float>(1, 2, 3));
    }
}

BOOST_AUTO_TEST_CASE(ImpulseResponseHelper_test)
{
    ImpulseResponseListFileTextHelper helper;

    boost::shared_ptr<HarkConfigFile> conf = helper.GetData();
    BOOST_CHECK(!conf);

    helper.AnalyzeLine("Version = 1.0");
    conf = helper.GetData();

    BOOST_CHECK(conf);
    if (conf)
    {
        BOOST_CHECK_EQUAL(conf->GetId(), "ImpulseResponseListFile");
        BOOST_CHECK(conf->GetVersion() == Version(1, 0));
    }

    boost::shared_ptr<ImpulseResponseListFile1_0> obj = static_pointer_cast<ImpulseResponseListFile1_0>(conf);

    BOOST_CHECK(obj);
    if (obj)
    {
        BOOST_CHECK_EQUAL(obj->GetSources().size(), 0);
    }

    helper.AnalyzeLine("1, 2, 3 %%hoge, 4");
    conf = helper.GetData();
    obj = static_pointer_cast<ImpulseResponseListFile1_0>(conf);

    BOOST_CHECK(obj);
    if (obj)
    {
        BOOST_CHECK_EQUAL(obj->GetSources().size(), 1);
        BOOST_CHECK(obj->GetSources()[0].GetPos() == Vector3<float>(1, 2, 3));
    }
}

BOOST_AUTO_TEST_CASE(Read_test)
{
    stringstream ss;
    ss.str(dataImpulseResponseListFile1_0);
    HarkConfigFileTextReader reader(ss);

    boost::shared_ptr<HarkConfigFile> conf = reader.Read();

    BOOST_CHECK(conf);
    if (conf)
    {
        BOOST_CHECK_EQUAL(conf->GetId(), "ImpulseResponseListFile");
        BOOST_CHECK(conf->GetVersion() == Version(1, 0));
    }

    boost::shared_ptr<ImpulseResponseListFile1_0> obj = static_pointer_cast<ImpulseResponseListFile1_0>(conf);
    BOOST_CHECK(obj);
    if (obj)
    {
        BOOST_CHECK_EQUAL(obj->GetSources().size(), 72);
        Source1 s = obj->GetSources().front();
        BOOST_CHECK(s.GetPos() == Vector3<float>(-0.9578, -0.0000, 0.2874));
        BOOST_CHECK_EQUAL(s.GetFile(), "/home/hoge/fltdir/flt_D180_00_ch*.flt");
        BOOST_CHECK_EQUAL(s.GetCutStart(), 1);
    }
}

struct IsExpectedAnalyzeError
{
    explicit IsExpectedAnalyzeError(const AnalyzeError& expected) :
        expected(expected)
    { }

    bool operator ()(const AnalyzeError& e)
    {
        return strcmp(e.what(), expected.what()) == 0 && e.line() == expected.line();
    }

private:
    AnalyzeError expected;
};

BOOST_AUTO_TEST_CASE(Read_exception_test1)
{
    stringstream ss;
    ss <<
        "ImpulseResponse-ListFile\n"
        "Version 1.0\n"
        "1, 2, abc %%hoge, 1\n";
    HarkConfigFileTextReader reader(ss);

    BOOST_CHECK_EXCEPTION(reader.Read(), AnalyzeError, IsExpectedAnalyzeError(AnalyzeError("invalid line", 3)));

}

BOOST_AUTO_TEST_CASE(Read_exception_test2)
{
    stringstream ss;
    ss <<
        "ImpulseResponse-ListFile\n"
        "Version 1.0\n"
        "1, 2, 3 %%hoge, 1.2\n";
    HarkConfigFileTextReader reader(ss);

    BOOST_CHECK_EXCEPTION(reader.Read(), AnalyzeError, IsExpectedAnalyzeError(AnalyzeError("invalid line", 3)));

}

